// Copyright (c) 1999, A.H. van den Boogaard

#include "ftstring.h"
#include <cctype>

using std::toupper;

int cmp_nocase( const std::string &s, const std::string &s2 )
/* string comparison, ignoring upper or lower case
   from Bjarne Stroustrup, "The C++ Programming Language" 3rd ed., p. 591 */
{
   std::string::const_iterator p = s.begin();
   std::string::const_iterator p2 = s2.begin();

   while ( p != s.end() && p2 != s2.end() )
   {
      if ( toupper(*p) != toupper(*p2) )
         return ( toupper(*p)<toupper(*p2) ) ? -1:1;
      ++p;
      ++p2;
   }

   return s2.size() - s.size();
}

std::string get_basename( const std::string &fullname )
{
/*  basename is the substring between the last appearance of '/', '\' or ':'
    (or the start of the string) and the first appearance of '.' thereafter
    (or the end of the string).
    If fullname is empty, or no basename is found, an empty string is returned */

   size_t sub1 = fullname.find_last_of( "/\\:" ); // the last char before the basename
   if ( sub1 == std::string::npos ) sub1 = 0;
   else sub1++;  // the first char of the basename

   size_t sub2 = fullname.find( ".", sub1 ); // the first char after the basename
   if ( sub2 == std::string::npos ) sub2 = fullname.size()+1;

   std::string basename( fullname, sub1, sub2-sub1 );

   return basename;
}

std::string get_dirname( const std::string &fullname )
{
/*  dirname is the substring up to and including the last appearance of '/', '\' or ':'
    (or the start of the string).
    If fullname is empty, or no dirname is found, an empty string is returned */

   size_t sub = fullname.find_last_of( "/\\:" );
// sub is the index of the last char before the basename

   if ( sub == std::string::npos ) sub = 0;
   else sub++;
// now sub is the number of characters to copy

   std::string dirname( fullname, 0, sub );

   return dirname;
}
