// Copyright (c) 1999, A.H. van den Boogaard

#ifndef MYCONTROL_H
#define MYCONTROL_H

#define FULL_NEWTON_RAPHSON     1
#define MODIFIED_NEWTON_RAPHSON 2

#include "control.h"
#include "mydomain.h"
#include "modal.h"

class LinearControl:public FeaTure::Control<MyDomain>
{
   public:
      LinearControl():FeaTure::Control<MyDomain>(){}
      ~LinearControl(){}

      void initialize( MyDomain &model, std::ostream &out );
      bool continuate( MyDomain &model, std::ostream &out, std::ostream &post,
                       int steps=-1 );
      bool need_post_file(){ return false; }
};

class EigenControl:public FeaTure::Control<MyDomain>
{
   private:
      size_t nr_frequencies;
      std::vector<Mode*> modes;
      size_t current_mode;
      double shift;

   public:
      EigenControl():FeaTure::Control<MyDomain>(),nr_frequencies(5),shift(0.0){}
      ~EigenControl();

      void initialize( MyDomain &model, std::ostream &out );
      bool continuate( MyDomain &model, std::ostream &out, std::ostream &post,
                       int steps=-1 );
      bool need_post_file(){ return false; }
      void set_nr_frequencies( size_t i ){ nr_frequencies = i; }
      void set_shift( double s ){ shift=s; }
};

class ExplicitControl:public FeaTure::IncrementalControl<MyDomain>
{
   private:
      FeaTure::DiagonalSysMat *DiagMassSystemptr;
      FeaTure::Vector       delta_u_1;

      void solve_increment( MyDomain &model, std::ostream& out, int loadset,
                            double size );
      void log_increment_nr( std::ostream &out );

   public:
      ExplicitControl();
      ~ExplicitControl();

      void initialize( MyDomain &model, std::ostream& out );
};

class MyIncrementalIterativeControl:public
                             FeaTure::IncrementalIterativeControl<MyDomain>
{
   public:
      MyIncrementalIterativeControl();
      ~MyIncrementalIterativeControl();

      void print_norms( std::ostream& out );
};

class NonLinearStaticControl:public
                             MyIncrementalIterativeControl
{
   private:
      FeaTure::SystemMat *Systemptr;

      void solve_increment( MyDomain &model, std::ostream& out, int loadset, double size );

   public:
      NonLinearStaticControl();
      ~NonLinearStaticControl();

      void initialize( MyDomain &model, std::ostream& out );
};

class NonLinearNewmarkControl:public
                              MyIncrementalIterativeControl
{
   private:
      double      Newmark_alpha, Newmark_delta;

      FeaTure::Vector    velocity, acceleration;
      FeaTure::SparseSysMat *Systemptr, *MassSystemptr, *DampSystemptr;

      void solve_increment( MyDomain &model, std::ostream& out, int loadset, double size );

   public:
      NonLinearNewmarkControl();
      ~NonLinearNewmarkControl();
      void set_Newmark( double alpha, double delta );

      void initialize( MyDomain &model, std::ostream& out );
};

#endif /* MYCONTROL_H */
