// Copyright (c) 1999,2003, A.H. van den Boogaard

#ifndef CARTVECT_H
#define CARTVECT_H

#include <iostream>

namespace FeaTure
{

class CartVector
{
  protected:
    double x[3];

  public:
    CartVector(){};
    CartVector( const CartVector& vec );
    CartVector( double x, double y, double z );
    void print() const;
    CartVector add(const CartVector& src2) const;
    CartVector selfadd(const CartVector& src2);
    CartVector sub(const CartVector& src2) const;
    CartVector selfsub(const CartVector& src2);
    CartVector mul(double val) const;
    CartVector selfmult(double scalar);
    void getValues( double &xx, double &yy, double &zz ) const;
    double dot(const CartVector& vec2) const;
    double normsq() const;
    double norm() const;
    double& operator[](int i){ return x[i]; } // zero-based component
    const double& operator[](int i) const { return x[i]; }
    double& operator()(int i){ return x[i-1]; } // one-based component
    const double& operator()(int i) const { return x[i-1]; }


  friend CartVector operator+(const CartVector& a, const CartVector& b)
                { return a.add(b); }
  friend CartVector operator+=(CartVector& a, const CartVector& b)
                { return a.selfadd(b); }
  friend CartVector operator-(const CartVector& a, const CartVector& b)
                { return a.sub(b); }
  friend CartVector operator-=(CartVector& a, const CartVector& b)
                { return a.selfsub(b); }
  friend CartVector operator*(const CartVector& a, double b)
                { return a.mul(b); }
  friend CartVector operator*(double b, const CartVector& a)
                { return a.mul(b); }
  friend double operator*(const CartVector& a, const CartVector& b)
                { return a.dot(b); }

  friend CartVector outer(const CartVector& a, const CartVector& b);
};

std::ostream& operator<<( std::ostream& os, const CartVector& vec );

class Direction:public CartVector
{
  public:
    Direction():CartVector(){};
    Direction( const Direction& dir ):CartVector( dir ){};
    Direction( const CartVector& vect );
    void set_dir( double x, double y, double z );
    void get_dir( double& x, double& y, double& z ) const;
    void print() const;
};

class Location:public CartVector
{
  public:
    Location( double xx=0, double yy=0, double zz=0 );
    Location( const CartVector& vect ):CartVector( vect ){}
    void set_location( double x, double y, double z );
    void get_location( double& x, double& y, double& z ) const;
    void print() const;
};

CartVector outer( const CartVector& a, const CartVector& b );

} // namespace FeaTure

#endif /*CARTVECT_H */
