// Copyright (c) 1999, A.H. van den Boogaard

#ifndef POSTPROC_H
#define POSTPROC_H

#include <set>

namespace FeaTure
{

class PostProc
{
   public:
      enum PostItem { coordinates, stresses, eps_plast_eq };

   private:
      std::set< int,std::less<int> >  post_set;
      /* second argument should be default
         but does not work in BC++ 5.02 */
      bool post_all, post_last;
      bool displa, incdis, temp, inctemp, stress, epspeq;

   public:
      PostProc():post_all(false),post_last(false),displa(false),incdis(false),
                 temp(false),inctemp(false),stress(false),epspeq(false){}
      ~PostProc(){}

      void add_step( int step ){ post_set.insert( step ); }
      void add_all_steps(){ post_all = true; }
      void add_last_step(){ post_last = true; }
      bool process( int step, bool is_last ) const
         { return post_all || ( post_last && is_last ) || post_set.count( step ); }
      bool any_selected() const { return post_all || post_last || !post_set.empty(); }
      void set_displa(){  displa  = true; }
      void set_inddis(){  incdis  = true; }
      void set_temp(){    temp    = true; }
      void set_inctemp(){ inctemp = true; }
      void set_stress(){  stress  = true; }
      void set_epspeq(){  epspeq  = true; }

      bool need_displa()  const { return displa;  }
      bool need_incdis()  const { return incdis;  }
      bool need_temp()    const { return temp;    }
      bool need_inctemp() const { return inctemp; }
      bool need_stress()  const { return stress;  }
      bool need_epspeq()  const { return epspeq;  }
};

} // namespace FeaTure

#endif /* POSTPROC_H */
