// Copyright (c) 2005, A.H. van den Boogaard

#ifndef SYSVEC_H
#define SYSVEC_H

#include "ftmatrix.h"
#include "dofs.h"
#include "ftexcept.h"

namespace FeaTure
{

class SystemVector
{
   private:
     Vector free_vec;
     Vector prescribed_vec;
     Vector fixed_vec;

   public:
     SystemVector(){}

     SystemVector( size_t nr_free, size_t nr_prescribed, size_t nr_fixed )
     {
       resize( nr_free, nr_prescribed, nr_fixed );
       clear();
     }

     SystemVector( const SystemVector& A )
     {
       free_vec = A.free_vec;
       prescribed_vec = A.prescribed_vec;
       fixed_vec = A.fixed_vec;
     }

     ~SystemVector(){}

     void resize( size_t nr_free, size_t nr_prescribed, size_t nr_fixed )
     {
       free_vec.resize(nr_free);
       prescribed_vec.resize(nr_prescribed);
       fixed_vec.resize(nr_fixed);
     }

     void clear()
     {
       free_vec.clear();
       prescribed_vec.clear();
       fixed_vec.clear();
     }

     SystemVector& operator+=( const SystemVector& A )
     {
       free_vec += A.free_vec;
       prescribed_vec += A.prescribed_vec;
       fixed_vec += A.fixed_vec;
       return *this;
     }

     SystemVector& operator-=( const SystemVector& A )
     {
       free_vec -= A.free_vec;
       prescribed_vec -= A.prescribed_vec;
       fixed_vec -= A.fixed_vec;
       return *this;
     }

     SystemVector& operator*=( double s )
     {
       free_vec *= s;
       prescribed_vec *= s;
       fixed_vec *= s;
       return *this;
     }

     double& operator()( const Dof& d )
     {
        if ( d.is_free() )
          return free_vec[d.get_global_dof()-1];
        else if ( d.is_prescribed() )
          return prescribed_vec[d.get_global_dof()-1];
        else if ( d.is_fixed() )
          return fixed_vec[d.get_global_dof()-1];
        else
          throw program_error( "unkown dof_fixed type in SystemVector" ); 
     }

     const double& operator()( const Dof& d ) const
     {
        if ( d.is_free() )
          return free_vec[d.get_global_dof()-1];
        else if ( d.is_prescribed() )
          return prescribed_vec[d.get_global_dof()-1];
        else if ( d.is_fixed() )
          return fixed_vec[d.get_global_dof()-1];
        else
          throw program_error( "unkown dof_fixed type in SystemVector" ); 
     }

     const Vector& free() const { return free_vec; }
     const Vector& prescribed() const { return prescribed_vec; }
     const Vector& fixed() const { return fixed_vec; }
};

} // namespace FeaTure

#endif // SYSVEC_H
