// Copyright (c) 1999,2003, A.H. van den Boogaard

#ifndef TRANSMAT_H
#define TRANSMAT_H

#include "ftmatrix.h"

namespace FeaTure
{

class TransformationMatrix
{
   enum Type { NO_INIT, UNIT, PERMUTATION, SPARSE };

   struct Item
   {
      size_t i;
      size_t j;
      double val;
   };

   private:
       size_t dim1, dim2;
       Type   type;
       Item   *a;
       int    *p;
       size_t nr_nonzeros;
       size_t filled;

   public:
      TransformationMatrix():dim1(0), dim2(0), type(NO_INIT), a(NULL), p(NULL),
                             nr_nonzeros(0), filled(0){}
      ~TransformationMatrix(){ delete [] a; delete [] p; }

      void setUnit( size_t dim ){ dim1 = dim2 = dim; type = UNIT; }
      void setPermutation( size_t dimen1, size_t dimen2, int *perm ){
         dim1 = dimen1, dim2=dimen2, type = PERMUTATION;
         delete [] p; p = perm; }
      void setSparse( size_t dimen1, size_t dimen2, size_t nz ){
         dim1 = dimen1; dim2=dimen2; nr_nonzeros = nz; type = SPARSE;
         delete [] a; a = new Item[nr_nonzeros]; }
      void put( size_t i, size_t j, double val );
      double get( size_t i, size_t j ) const;

      Vector mult( const Vector &v ) const;
      Vector tmult( const Vector &v ) const;
      Matrix preTmult_postMult( const Matrix &M ) const;

      size_t num_rows() const { return dim1; }
      size_t num_cols() const { return dim2; }
};

std::ostream& operator<<(std::ostream &s, const TransformationMatrix &A);

inline Vector operator*(const TransformationMatrix &A,const Vector &B)
{
    return A.mult(B);
}

} // namespace FeaTure

#endif // TRANSMAT_H
