// Copyright (c) 2003, A.H. van den Boogaard
//
// test program for FeaTure::Matrix and FeaTure::Vector classes
// and related utility functions

#include "ftmatrix.h"
#include <iomanip>
#include <iostream>

using FeaTure::Matrix;
using FeaTure::Vector;
using std::cout;
using std::setw;
using std::endl;

extern "C" {
void dscal_( int*, double*, double*, int* );
}

int main()
{
   cout.precision(7);


   /*********** CONSTRUCTORS ***********/

   cout << "constructors and element access [] and ()" << endl << endl;

   Vector d;
   cout << "d.size() " << d.size() << endl << endl;

   Vector a(3);
   a[0] = 3.5;
   a[1] = 2.0;
   a[2] = 1.5;
   cout << "a(3) [] access" << endl << setw(5) << a << endl;

   Vector b(3,0);
   b(1) = a[0] + 1.5;
   b(3) = a(2) + 5.0;
   cout << "b(3,0) () access" << endl << setw(5) << b << endl;

   Vector e(a);
   cout << "e(a) copy constructor" << endl << setw(5) << e << endl;

   Matrix M(a);
   cout << "M(a) Matrix constructor from Vector" << endl
        << setw(5) << M << endl;

   Vector c(M);
   cout << "c(M) Vector constructor from Matrix; c.print_row(cout)" << setw(5);
   c.print_row( cout );
   cout << endl << endl;

   Matrix C;
   cout << "C.size1() " << C.size1() << " C.size2() " << C.size2()
        << endl << endl;

   Matrix A(2,2);
   
   A(1,1) = 2.0;
   A(1,2) = 1.0;
   A(2,1) = 0.5;
   A(2,2) = 1.5;
   cout << "A(2,2) () access" << endl << setw(5) << A << endl;

   Matrix B(2,3,0.0);
   
   B[0][0] = A(1,1);
   B[0][1] = A[0][1];
   B[1][0] = 1.5;
   B[1][1] = 0.5;
   cout << "B(2,3,0.0) [] access" << endl << setw(5) << B << endl;

   Matrix G(A);
   cout << "G(A) copy constructor" << endl << setw(5) << G << endl;


   /*********** ASSIGNMENTS ***********/

   cout << "assignments" << endl << endl;

   C = A;
   cout << "assignment C=A" << endl << setw(7) << C << endl;

   c = a;
   cout << "assignment c=a" << endl << setw(7) << c << endl;


   /*********** CLEAR, RESIZE ***********/

   cout << "clear and resize" << endl << endl;

   d.resize(5);
   d.clear();
   cout << "d.resize(5); d.clear();" << endl << setw(5) << d << endl;

   C.resize(4,4);
   C.clear();
   cout << "C.resize(4,4); C.clear();" << endl << setw(5) << C << endl;


   /*********** ADD SUBMATRIX, SUBVECTOR, CONDENSE ***********/

   cout << "add submatrix, add subvector, condense matrix row column"
        << endl << endl;

   d.add_SubVec( 1, a );
   d.add_SubVec( 3, b );
   cout << "d.add_SubVec(1,a); d.add_SubVec(3,b);" << endl
        << setw(5) << d << endl;

   C.add_SubMat( 1, 1, A );
   C.add_SubMat( 2, 2, B );
   cout << "C.add_SubMat(1,1,A); C.add_SubMat(2,2,B);" << endl
        << setw(5) << C << endl;

   Matrix D( C.without_rowcol( 3 ) );
   cout << "D(C.without_rowcol(3))" << endl << setw(5) << D << endl;


   /*********** INVERSE, DETERMINANT, NORM ***********/

   cout << "inverse matrix, deteminant" << endl << endl;

   Matrix E(A.inv());
   cout << "E(A.inv())" << endl << setw(14) << E << endl;
   cout << "E*A" << endl << setw(14) << E*A << endl;

   D(1,3) = 0.5;
   D(2,3) = 1.5;
   D(3,1) = 0.25;
   D(3,2) = 1.0;
   D(3,3) = 3.0;
   cout << "D" << endl << setw(5) << D << endl;
   cout << "D*D.inv()" << endl << setw(14) << D*D.inv() << endl;

   Matrix F(5,5,0.0);
   F.add_SubMat(1,1,D);
   F.add_SubMat(3,3,D);
   cout << "F" << endl << setw(5) << F << endl;
   cout << "F*F.inv()" << endl << setw(14) << F*F.inv() << endl;
  
   cout << "A.determinant()) " << A.determinant() << endl;
   cout << endl;

   cout << "D.determinant()) " << D.determinant() << endl;
   cout << endl;
  
   cout << "vector norm" << endl << endl;

   cout << "d.norm() " << d.norm() << endl << endl;

   G = B;

   /*********** MEMBER OPERATORS +=, -= and *= ***********/

   cout << "+=, -=, *= operators" << endl << endl;

   B = A;
   B += A;
   cout << "B+=A" << endl << setw(5) << B << endl;
   B -= A;
   cout << "B-=A" << endl << setw(5) << B << endl;
   B *= 2.0;
   cout << "B*=2.0" << endl << setw(5) << B << endl;

   b = a;
   b += a;
   cout << "b+=a" << endl << setw(5) << b << endl;
   b -= a;
   cout << "b-=a" << endl << setw(5) << b << endl;
   b *= 2.0;
   cout << "b*=2.0" << endl << setw(5) << b << endl;

   B = G;


   /*********** TRANSPOSE FUNCTIONS ***********/

   cout << "Matrix and Vector transpose" << endl << endl;

   cout << "~B (transpose of Matrix B)" << endl << setw(5) << ~B << endl;

   cout << "~c (transpose of Vector c)" << endl << setw(5) << ~c << endl;


   /*********** MATRIX-MATRIX MATRIX-VECTOR MULTIPLICATIONS ***********/

   cout << "Matrix and Vector multiplication functions" << endl << endl;

   cout << "mult(A,B) A*B" << endl << setw(5) << mult(A,B) << endl;

   cout << "mult(D,c) D*c" << endl << setw(5) << mult(D,c) << endl;

   cout << "tmult(B,A) Bt*A" << endl << setw(5) << tmult(B,A) << endl;

   cout << "tmult(D,c) Dt*c" << endl << setw(5) << tmult(D,c) << endl;

   cout << "tmult(c,D) ct*D" << endl << setw(5) << tmult(c,D) << endl;

   double val;
   val = inner_product( a, b );
   cout << "val = inner_product(a,b) " << val << endl;
   val = inner_product( a, b, val );
   cout << "val = inner_product(a,b,val) " << val << endl;

   C = A*B;
   cout << "C=A*B" << endl << setw(7) << C << endl;

   cout << "D*c" << endl << setw(5) << D*c << endl;

   Matrix Mt( ~M );
   cout << "a*Mt" << endl << setw(5) << a*Mt << endl;

   cout << "2.0*(~B*A*B)" << endl << setw(7) << 2.0*(~B*A*B) << endl;
   D.clear();
   Add_sAtBA( D, 2.0, B, A );
   cout << "Add_sAtBA(D,2.0,B,A)" << endl << setw(7) << D << endl;
   Add_sAtBA( D, 1.0, B, A );
   cout << "Add_sAtBA(D,1.0,B,A)" << endl << setw(7) << D << endl;

   cout << "Scalar multiplications" << endl;

   cout << "a" << endl << setw(5) << a << endl;
   cout << "2*a" << endl << setw(5) << 2*a << endl;
   cout << "a*2" << endl << setw(5) << a*2 << endl;
   cout << "A" << endl << setw(5) << A << endl;
   cout << "2*A" << endl << setw(5) << 2*A << endl;
   cout << "A*2" << endl << setw(5) << A*2 << endl;

   cout << "+ and - operators" << endl << endl;

   B = A;
   C = A+B;
   cout << "C=A+B" << endl << setw(5) << C << endl;
   D = C-A;
   cout << "D=C-A" << endl << setw(5) << D << endl;

   b = a;
   c = a+b;
   cout << "c=a+b" << endl << setw(5) << c << endl;
   cout << "c-a" << endl << setw(5) << (c-a) << endl;

   cout << "Mixed Matrix/Vector addition and substraction" << endl << endl;
   D = a;
   cout << "D=a" << endl << D << endl;
   c = a+D;
   cout << "c=a+D" << endl << setw(5) << c << endl;
   cout << "c-D" << endl << setw(5) << (c-D) << endl;
   c = D+a;
   cout << "c=D+a" << endl << setw(5) << c << endl;
   cout << "D-c" << endl << setw(5) << (D-c) << endl;

   return 0;
}
